@@ -2,23 +2,13 @@
 # -*- coding: utf-8 -*-
 
 # (c) 2012, Michael DeHaan <michael.dehaan@gmail.com>
-#
-# This file is part of Ansible
-#
-# Ansible is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# Ansible is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
+# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
-ANSIBLE_METADATA = {'metadata_version': '1.0',
+from __future__ import absolute_import, division, print_function
+__metaclass__ = type
+
+
+ANSIBLE_METADATA = {'metadata_version': '1.1',
                     'status': ['preview'],
                     'supported_by': 'core'}
 
@@ -42,10 +32,8 @@ options:
     dest:
         required: true
         description:
-            - Absolute path of where the repository should be checked out to.
-              This parameter is required, unless C(clone) is set to C(no)
-              This change was made in version 1.8.3. Prior to this version,
-              the C(dest) parameter was always required.
+            - The path of where the repository should be checked out. This
+              parameter is required, unless C(clone) is set to C(no).
     version:
         required: false
         default: "HEAD"
@@ -60,9 +48,8 @@ options:
         choices: [ "yes", "no" ]
         version_added: "1.5"
         description:
-            - if C(yes), adds the hostkey for the repo url if not already
-              added. If ssh_opts contains "-o StrictHostKeyChecking=no",
-              this parameter is ignored.
+            - if C(yes), ensure that "-o StrictHostKeyChecking=no" is
+              present as an ssh options.
     ssh_opts:
         required: false
         default: None
@@ -184,7 +171,16 @@ options:
             - if C(yes), when cloning or checking out a C(version) verify the
               signature of a GPG signed commit. This requires C(git) version>=2.1.0
               to be installed. The commit MUST be signed and the public key MUST
-              be trusted in the GPG trustdb.
+              be present in the GPG keyring.
+
+    archive:
+        required: false
+        version_added: "2.4"
+        description:
+            - Specify archive file path with extension. If specified, creates an
+              archive file of the specified format containing the tree structure
+              for the source tree.
+              Allowed archive formats ["zip", "tar.gz", "tar", "tgz"]
 
 requirements:
     - git>=1.7.1 (the command line tool)
@@ -200,7 +196,7 @@ notes:
 EXAMPLES = '''
 # Example git checkout from Ansible Playbooks
 - git:
-    repo: git://foosball.example.org/path/to/repo.git
+    repo: 'https://foosball.example.org/path/to/repo.git'
     dest: /srv/checkout
     version: release-0.22
 
@@ -211,14 +207,14 @@ EXAMPLES = '''
 
 # Example just ensuring the repo checkout exists
 - git:
-    repo: git://foosball.example.org/path/to/repo.git
+    repo: 'https://foosball.example.org/path/to/repo.git'
     dest: /srv/checkout
     update: no
 
 # Example just get information about the repository whether or not it has
 # already been cloned locally.
 - git:
-    repo: git://foosball.example.org/path/to/repo.git
+    repo: 'https://foosball.example.org/path/to/repo.git'
     dest: /srv/checkout
     clone: no
     update: no
@@ -228,11 +224,18 @@ EXAMPLES = '''
     repo: https://github.com/ansible/ansible-examples.git
     dest: /src/ansible-examples
     refspec: '+refs/pull/*:refs/heads/*'
+
+# Example Create git archive from repo
+- git:
+    repo: https://github.com/ansible/ansible-examples.git
+    dest: /src/ansible-examples
+    archive: /tmp/ansible-examples.zip
+
 '''
 
 RETURN = '''
 after:
-    description: last commit revision of the repository retrived during the update
+    description: last commit revision of the repository retrieved during the update
     returned: success
     type: string
     sample: 4c020102a9cd6fe908c9a4a326a38f972f63a903
@@ -253,16 +256,17 @@ warnings:
     sample: Your git version is too old to fully support the depth argument. Falling back to full checkouts.
 '''
 
+import filecmp
 import os
 import re
 import shlex
 import stat
 import sys
+import shutil
 import tempfile
 from distutils.version import LooseVersion
 
 from ansible.module_utils.basic import AnsibleModule, get_module_path
-from ansible.module_utils.known_hosts import add_git_host_key
 from ansible.module_utils.six import b, string_types
 from ansible.module_utils._text import to_native
 
@@ -286,7 +290,7 @@ def head_splitter(headfile, remote, module=None, fail_on_error=False):
                 rawdata = rawdata.replace('refs/remotes/%s' % remote, '', 1)
                 refparts = rawdata.split(' ')
                 newref = refparts[-1]
-                nrefparts = newref.split('/',2)
+                nrefparts = newref.split('/', 2)
                 res = nrefparts[-1].rstrip('\n')
             except:
                 if fail_on_error and module:
@@ -301,11 +305,11 @@ def unfrackgitpath(path):
     # copied from ansible.utils.path
     return os.path.normpath(os.path.realpath(os.path.expanduser(os.path.expandvars(path))))
 
-def get_submodule_update_params(module, git_path, cwd):
 
-    #or: git submodule [--quiet] update [--init] [-N|--no-fetch]
-    #[-f|--force] [--rebase] [--reference <repository>] [--merge]
-    #[--recursive] [--] [<path>...]
+def get_submodule_update_params(module, git_path, cwd):
+    # or: git submodule [--quiet] update [--init] [-N|--no-fetch]
+    # [-f|--force] [--rebase] [--reference <repository>] [--merge]
+    # [--recursive] [--] [<path>...]
 
     params = []
 
@@ -318,9 +322,9 @@ def get_submodule_update_params(module, git_path, cwd):
         if 'git submodule [--quiet] update ' in line:
             update_line = line
     if update_line:
-        update_line = update_line.replace('[','')
-        update_line = update_line.replace(']','')
-        update_line = update_line.replace('|',' ')
+        update_line = update_line.replace('[', '')
+        update_line = update_line.replace(']', '')
+        update_line = update_line.replace('|', ' ')
         parts = shlex.split(update_line)
         for part in parts:
             if part.startswith('--'):
@@ -329,12 +333,13 @@ def get_submodule_update_params(module, git_path, cwd):
 
     return params
 
+
 def write_ssh_wrapper():
     module_dir = get_module_path()
     try:
         # make sure we have full permission to the module_dir, which
         # may not be the case if we're sudo'ing to a non-root user
-        if os.access(module_dir, os.W_OK|os.R_OK|os.X_OK):
+        if os.access(module_dir, os.W_OK | os.R_OK | os.X_OK):
             fd, wrapper_path = tempfile.mkstemp(prefix=module_dir + '/')
         else:
             raise OSError
@@ -348,6 +353,9 @@ else
     BASEOPTS=$GIT_SSH_OPTS
 fi
 
+# Let ssh fail rather than prompt
+BASEOPTS="$BASEOPTS -o BatchMode=yes"
+
 if [ -z "$GIT_KEY" ]; then
     ssh $BASEOPTS "$@"
 else
@@ -360,6 +368,7 @@ fi
     os.chmod(wrapper_path, st.st_mode | stat.S_IEXEC)
     return wrapper_path
 
+
 def set_git_ssh(ssh_wrapper, key_file, ssh_opts):
 
     if os.environ.get("GIT_SSH"):
@@ -378,6 +387,7 @@ def set_git_ssh(ssh_wrapper, key_file, ssh_opts):
     if ssh_opts:
         os.environ["GIT_SSH_OPTS"] = ssh_opts
 
+
 def get_version(module, git_path, dest, ref="HEAD"):
     ''' samples the version of the git repo '''
 
@@ -386,11 +396,16 @@ def get_version(module, git_path, dest, ref="HEAD"):
     sha = to_native(stdout).rstrip('\n')
     return sha
 
+
 def get_submodule_versions(git_path, module, dest, version='HEAD'):
     cmd = [git_path, 'submodule', 'foreach', git_path, 'rev-parse', version]
     (rc, out, err) = module.run_command(cmd, cwd=dest)
     if rc != 0:
-        module.fail_json(msg='Unable to determine hashes of submodules', stdout=out, stderr=err, rc=rc)
+        module.fail_json(
+            msg='Unable to determine hashes of submodules',
+            stdout=out,
+            stderr=err,
+            rc=rc)
     submodules = {}
     subm_name = None
     for line in out.splitlines():
@@ -408,6 +423,7 @@ def get_submodule_versions(git_path, module, dest, version='HEAD'):
 
     return submodules
 
+
 def clone(git_path, module, repo, dest, remote, depth, version, bare,
           reference, refspec, verify_commit):
     ''' makes a new git repo if it does not already exist '''
@@ -417,18 +433,18 @@ def clone(git_path, module, repo, dest, remote, depth, version, bare,
         os.makedirs(dest_dirname)
     except:
         pass
-    cmd = [ git_path, 'clone' ]
+    cmd = [git_path, 'clone']
 
     if bare:
         cmd.append('--bare')
     else:
-        cmd.extend([ '--origin', remote ])
+        cmd.extend(['--origin', remote])
     if depth:
         if version == 'HEAD' or refspec:
-            cmd.extend([ '--depth', str(depth) ])
+            cmd.extend(['--depth', str(depth)])
         elif is_remote_branch(git_path, module, dest, repo, version) \
                 or is_remote_tag(git_path, module, dest, repo, version):
-            cmd.extend([ '--depth', str(depth) ])
+            cmd.extend(['--depth', str(depth)])
             cmd.extend(['--branch', version])
         else:
             # only use depth if the remote object is branch or tag (i.e. fetchable)
@@ -436,8 +452,8 @@ def clone(git_path, module, repo, dest, remote, depth, version, bare,
                         "Shallow clones are only available for "
                         "HEAD, branches, tags or in combination with refspec.")
     if reference:
-        cmd.extend([ '--reference', str(reference) ])
-    cmd.extend([ repo, dest ])
+        cmd.extend(['--reference', str(reference)])
+    cmd.extend([repo, dest])
     module.run_command(cmd, check_rc=True, cwd=dest_dirname)
     if bare:
         if remote != 'origin':
@@ -446,13 +462,14 @@ def clone(git_path, module, repo, dest, remote, depth, version, bare,
     if refspec:
         cmd = [git_path, 'fetch']
         if depth:
-            cmd.extend([ '--depth', str(depth) ])
+            cmd.extend(['--depth', str(depth)])
         cmd.extend([remote, refspec])
         module.run_command(cmd, check_rc=True, cwd=dest)
 
     if verify_commit:
         verify_commit_sign(git_path, module, dest, version)
 
+
 def has_local_mods(module, git_path, dest, bare):
     if bare:
         return False
@@ -464,6 +481,7 @@ def has_local_mods(module, git_path, dest, bare):
 
     return len(lines) > 0
 
+
 def reset(git_path, module, dest):
     '''
     Resets the index and working tree to HEAD.
@@ -473,10 +491,11 @@ def reset(git_path, module, dest):
     cmd = "%s reset --hard HEAD" % (git_path,)
     return module.run_command(cmd, check_rc=True, cwd=dest)
 
+
 def get_diff(module, git_path, dest, repo, remote, depth, bare, before, after):
     ''' Return the difference between 2 versions '''
     if before is None:
-        return { 'prepared': '>> Newly checked out %s' % after }
+        return {'prepared': '>> Newly checked out %s' % after}
     elif before != after:
         # Ensure we have the object we are referring to during git diff !
         git_version_used = git_version(git_path, module)
@@ -484,15 +503,16 @@ def get_diff(module, git_path, dest, repo, remote, depth, bare, before, after):
         cmd = '%s diff %s %s' % (git_path, before, after)
         (rc, out, err) = module.run_command(cmd, cwd=dest)
         if rc == 0 and out:
-            return { 'prepared': out }
+            return {'prepared': out}
         elif rc == 0:
-            return { 'prepared': '>> No visual differences between %s and %s' % (before, after) }
+            return {'prepared': '>> No visual differences between %s and %s' % (before, after)}
         elif err:
-            return { 'prepared': '>> Failed to get proper diff between %s and %s:\n>> %s' % (before, after, err) }
+            return {'prepared': '>> Failed to get proper diff between %s and %s:\n>> %s' % (before, after, err)}
         else:
-            return { 'prepared': '>> Failed to get proper diff between %s and %s' % (before, after) }
+            return {'prepared': '>> Failed to get proper diff between %s and %s' % (before, after)}
     return {}
 
+
 def get_remote_head(git_path, module, dest, version, remote, bare):
     cloning = False
     cwd = None
@@ -535,6 +555,7 @@ def get_remote_head(git_path, module, dest, version, remote, bare):
     rev = out.split()[0]
     return rev
 
+
 def is_remote_tag(git_path, module, dest, remote, version):
     cmd = '%s ls-remote %s -t refs/tags/%s' % (git_path, remote, version)
     (rc, out, err) = module.run_command(cmd, check_rc=True, cwd=dest)
@@ -543,6 +564,7 @@ def is_remote_tag(git_path, module, dest, remote, version):
     else:
         return False
 
+
 def get_branches(git_path, module, dest):
     branches = []
     cmd = '%s branch --no-color -a' % (git_path,)
@@ -554,17 +576,21 @@ def get_branches(git_path, module, dest):
             branches.append(line.strip())
     return branches
 
-def get_tags(git_path, module, dest):
+
+def get_annotated_tags(git_path, module, dest):
     tags = []
-    cmd = '%s tag' % (git_path,)
+    cmd = [git_path, 'for-each-ref', 'refs/tags/', '--format', '%(objecttype):%(refname:short)']
     (rc, out, err) = module.run_command(cmd, cwd=dest)
     if rc != 0:
         module.fail_json(msg="Could not determine tag data - received %s" % out, stdout=out, stderr=err)
     for line in to_native(out).split('\n'):
         if line.strip():
-            tags.append(line.strip())
+            tagtype, tagname = line.strip().split(':')
+            if tagtype == 'tag':
+                tags.append(tagname)
     return tags
 
+
 def is_remote_branch(git_path, module, dest, remote, version):
     cmd = '%s ls-remote %s -h refs/heads/%s' % (git_path, remote, version)
     (rc, out, err) = module.run_command(cmd, check_rc=True, cwd=dest)
@@ -573,6 +599,7 @@ def is_remote_branch(git_path, module, dest, remote, version):
     else:
         return False
 
+
 def is_local_branch(git_path, module, dest, branch):
     branches = get_branches(git_path, module, dest)
     lbranch = '%s' % branch
@@ -583,6 +610,7 @@ def is_local_branch(git_path, module, dest, branch):
     else:
         return False
 
+
 def is_not_a_branch(git_path, module, dest):
     branches = get_branches(git_path, module, dest)
     for branch in branches:
@@ -590,6 +618,7 @@ def is_not_a_branch(git_path, module, dest):
             return True
     return False
 
+
 def get_head_branch(git_path, module, dest, remote, bare=False):
     '''
     Determine what branch HEAD is associated with.  This is partly
@@ -632,6 +661,7 @@ def get_head_branch(git_path, module, dest, remote, bare=False):
     branch = head_splitter(headfile, remote, module=module, fail_on_error=True)
     return branch
 
+
 def get_remote_url(git_path, module, dest, remote):
     '''Return URL of remote source for repo.'''
     command = [git_path, 'ls-remote', '--get-url', remote]
@@ -642,6 +672,7 @@ def get_remote_url(git_path, module, dest, remote):
         return None
     return to_native(out).rstrip('\n')
 
+
 def set_remote_url(git_path, module, repo, dest, remote):
     ''' updates repo from remote sources '''
     # Return if remote URL isn't changing.
@@ -659,6 +690,7 @@ def set_remote_url(git_path, module, repo, dest, remote):
     # for Git versions prior to 1.7.5 that lack required functionality.
     return remote_url is not None
 
+
 def fetch(git_path, module, repo, dest, version, remote, depth, bare, refspec, git_version_used):
     ''' updates repo from remote sources '''
     set_remote_url(git_path, module, repo, dest, remote)
@@ -686,7 +718,7 @@ def fetch(git_path, module, repo, dest, version, remote, depth, bare, refspec, g
             else:
                 refspecs.append(version)
         elif is_remote_tag(git_path, module, dest, repo, version):
-            refspecs.append('+refs/tags/'+version+':refs/tags/'+version)
+            refspecs.append('+refs/tags/' + version + ':refs/tags/' + version)
         if refspecs:
             # if refspecs is empty, i.e. version is neither heads nor tags
             # assume it is a version hash
@@ -713,11 +745,12 @@ def fetch(git_path, module, repo, dest, version, remote, depth, bare, refspec, g
 
     commands.append((fetch_str, fetch_cmd + refspecs))
 
-    for (label,command) in commands:
-        (rc,out,err) = module.run_command(command, cwd=dest)
+    for (label, command) in commands:
+        (rc, out, err) = module.run_command(command, cwd=dest)
         if rc != 0:
             module.fail_json(msg="Failed to %s: %s %s" % (label, out, err), cmd=command)
 
+
 def submodules_fetch(git_path, module, remote, track_submodules, dest):
     changed = False
 
@@ -734,15 +767,6 @@ def submodules_fetch(git_path, module, remote, track_submodules, dest):
             if not os.path.exists(os.path.join(dest, path, '.git')):
                 changed = True
 
-        # add the submodule repo's hostkey
-        if line.strip().startswith('url'):
-            repo = line.split('=', 1)[1].strip()
-            if module.params['ssh_opts'] is not None:
-                if "-o StrictHostKeyChecking=no" not in module.params['ssh_opts']:
-                    add_git_host_key(module, repo, accept_hostkey=module.params['accept_hostkey'])
-            else:
-                add_git_host_key(module, repo, accept_hostkey=module.params['accept_hostkey'])
-
     # Check for updates to existing modules
     if not changed:
         # Fetch updates
@@ -754,10 +778,9 @@ def submodules_fetch(git_path, module, remote, track_submodules, dest):
 
         if track_submodules:
             # Compare against submodule HEAD
-            ### FIXME: determine this from .gitmodules
+            # FIXME: determine this from .gitmodules
             version = 'master'
-            after = get_submodule_versions(git_path, module, dest, '%s/%s'
-                    % (remote, version))
+            after = get_submodule_versions(git_path, module, dest, '%s/%s' % (remote, version))
             if begin != after:
                 changed = True
         else:
@@ -772,6 +795,7 @@ def submodules_fetch(git_path, module, remote, track_submodules, dest):
                     break
     return changed
 
+
 def submodule_update(git_path, module, dest, track_submodules, force=False):
     ''' init and update any submodules '''
 
@@ -781,12 +805,12 @@ def submodule_update(git_path, module, dest, track_submodules, force=False):
     # skip submodule commands if .gitmodules is not present
     if not os.path.exists(os.path.join(dest, '.gitmodules')):
         return (0, '', '')
-    cmd = [ git_path, 'submodule', 'sync' ]
+    cmd = [git_path, 'submodule', 'sync']
     (rc, out, err) = module.run_command(cmd, check_rc=True, cwd=dest)
     if 'remote' in params and track_submodules:
-        cmd = [ git_path, 'submodule', 'update', '--init', '--recursive' ,'--remote' ]
+        cmd = [git_path, 'submodule', 'update', '--init', '--recursive', '--remote']
     else:
-        cmd = [ git_path, 'submodule', 'update', '--init', '--recursive' ]
+        cmd = [git_path, 'submodule', 'update', '--init', '--recursive']
     if force:
         cmd.append('--force')
     (rc, out, err) = module.run_command(cmd, cwd=dest)
@@ -794,6 +818,7 @@ def submodule_update(git_path, module, dest, track_submodules, force=False):
         module.fail_json(msg="Failed to init/update submodules: %s" % out + err)
     return (rc, out, err)
 
+
 def set_remote_branch(git_path, module, dest, remote, version, depth):
     """set refs for the remote branch version
 
@@ -808,6 +833,7 @@ def set_remote_branch(git_path, module, dest, remote, version, depth):
     if rc != 0:
         module.fail_json(msg="Failed to fetch branch from remote: %s" % version, stdout=out, stderr=err, rc=rc)
 
+
 def switch_version(git_path, module, dest, remote, version, verify_commit, depth):
     cmd = ''
     if version == 'HEAD':
@@ -816,7 +842,7 @@ def switch_version(git_path, module, dest, remote, version, verify_commit, depth
         if rc != 0:
             module.fail_json(msg="Failed to checkout branch %s" % branch,
                              stdout=out, stderr=err, rc=rc)
-        cmd = "%s reset --hard %s" % (git_path, remote)
+        cmd = "%s reset --hard %s/%s --" % (git_path, remote, branch)
     else:
         # FIXME check for local_branch first, should have been fetched already
         if is_remote_branch(git_path, module, dest, remote, version):
@@ -830,8 +856,7 @@ def switch_version(git_path, module, dest, remote, version, verify_commit, depth
             else:
                 (rc, out, err) = module.run_command("%s checkout --force %s" % (git_path, version), cwd=dest)
                 if rc != 0:
-                    module.fail_json(msg="Failed to checkout branch %s" % version,
-                                     stdout=out, stderr=err, rc=rc)
+                    module.fail_json(msg="Failed to checkout branch %s" % version, stdout=out, stderr=err, rc=rc)
                 cmd = "%s reset --hard %s/%s" % (git_path, remote, version)
         else:
             cmd = "%s checkout --force %s" % (git_path, version)
@@ -851,7 +876,7 @@ def switch_version(git_path, module, dest, remote, version, verify_commit, depth
 
 
 def verify_commit_sign(git_path, module, dest, version):
-    if version in get_tags(git_path, module, dest):
+    if version in get_annotated_tags(git_path, module, dest):
         git_sub = "verify-tag"
     else:
         git_sub = "verify-commit"
@@ -867,7 +892,8 @@ def git_version(git_path, module):
     cmd = "%s --version" % git_path
     (rc, out, err) = module.run_command(cmd)
     if rc != 0:
-        # one could fail_json here, but the version info is not that important, so let's try to fail only on actual git commands
+        # one could fail_json here, but the version info is not that important,
+        # so let's try to fail only on actual git commands
         return None
     rematch = re.search('git version (.*)$', to_native(out))
     if not rematch:
@@ -875,11 +901,72 @@ def git_version(git_path, module):
     return LooseVersion(rematch.groups()[0])
 
 
+def git_archive(git_path, module, dest, archive, archive_fmt, version):
+    """ Create git archive in given source directory """
+    cmd = "%s archive --format=%s --output=%s %s" \
+          % (git_path, archive_fmt, archive, version)
+    (rc, out, err) = module.run_command(cmd, cwd=dest)
+    if rc != 0:
+        module.fail_json(msg="Failed to perform archive operation",
+                         details="Git archive command failed to create "
+                                 "archive %s using %s directory."
+                                 "Error: %s" % (archive, dest, err))
+    return rc, out, err
+
+
+def create_archive(git_path, module, dest, archive, version, repo, result):
+    """ Helper function for creating archive using git_archive """
+    all_archive_fmt = {'.zip': 'zip', '.gz': 'tar.gz', '.tar': 'tar',
+                       '.tgz': 'tgz'}
+    _, archive_ext = os.path.splitext(archive)
+    archive_fmt = all_archive_fmt.get(archive_ext, None)
+    if archive_fmt is None:
+        module.fail_json(msg="Unable to get file extension from "
+                             "archive file name : %s" % archive,
+                         details="Please specify archive as filename with "
+                                 "extension. File extension can be one "
+                                 "of ['tar', 'tar.gz', 'zip', 'tgz']")
+
+    repo_name = repo.split("/")[-1].replace(".git", "")
+
+    if os.path.exists(archive):
+        # If git archive file exists, then compare it with new git archive file.
+        # if match, do nothing
+        # if does not match, then replace existing with temp archive file.
+        tempdir = tempfile.mkdtemp()
+        new_archive_dest = os.path.join(tempdir, repo_name)
+        new_archive = new_archive_dest + '.' + archive_fmt
+        git_archive(git_path, module, dest, new_archive, archive_fmt, version)
+
+        # filecmp is supposed to be efficient than md5sum checksum
+        if filecmp.cmp(new_archive, archive):
+            result.update(changed=False)
+            # Cleanup before exiting
+            try:
+                shutil.remove(tempdir)
+            except OSError:
+                pass
+        else:
+            try:
+                shutil.move(new_archive, archive)
+                shutil.remove(tempdir)
+                result.update(changed=True)
+            except OSError as e:
+                module.fail_json(msg="Failed to move %s to %s" %
+                                     (new_archive, archive),
+                                 details="Error occured while moving : %s"
+                                         % to_native(e))
+    else:
+        # Perform archive from local directory
+        git_archive(git_path, module, dest, archive, archive_fmt, version)
+        result.update(changed=True)
+
+
 # ===========================================
 
 def main():
     module = AnsibleModule(
-        argument_spec = dict(
+        argument_spec=dict(
             dest=dict(type='path'),
             repo=dict(required=True, aliases=['name']),
             version=dict(default='HEAD'),
@@ -899,28 +986,37 @@ def main():
             recursive=dict(default='yes', type='bool'),
             track_submodules=dict(default='no', type='bool'),
             umask=dict(default=None, type='raw'),
+            archive=dict(type='path'),
         ),
         supports_check_mode=True
     )
 
-    dest      = module.params['dest']
-    repo      = module.params['repo']
-    version   = module.params['version']
-    remote    = module.params['remote']
-    refspec   = module.params['refspec']
-    force     = module.params['force']
-    depth     = module.params['depth']
-    update    = module.params['update']
+    dest = module.params['dest']
+    repo = module.params['repo']
+    version = module.params['version']
+    remote = module.params['remote']
+    refspec = module.params['refspec']
+    force = module.params['force']
+    depth = module.params['depth']
+    update = module.params['update']
     allow_clone = module.params['clone']
-    bare      = module.params['bare']
+    bare = module.params['bare']
     verify_commit = module.params['verify_commit']
     reference = module.params['reference']
-    git_path  = module.params['executable'] or module.get_bin_path('git', True)
-    key_file  = module.params['key_file']
-    ssh_opts  = module.params['ssh_opts']
-    umask  = module.params['umask']
+    git_path = module.params['executable'] or module.get_bin_path('git', True)
+    key_file = module.params['key_file']
+    ssh_opts = module.params['ssh_opts']
+    umask = module.params['umask']
+    archive = module.params['archive']
 
-    result = dict(changed = False, warnings=list())
+    result = dict(changed=False, warnings=list())
+
+    if module.params['accept_hostkey']:
+        if ssh_opts is not None:
+            if "-o StrictHostKeyChecking=no" not in ssh_opts:
+                ssh_opts += " -o StrictHostKeyChecking=no"
+        else:
+            ssh_opts = "-o StrictHostKeyChecking=no"
 
     # evaluate and set the umask before doing anything else
     if umask is not None:
@@ -955,18 +1051,10 @@ def main():
     # create a wrapper script and export
     # GIT_SSH=<path> as an environment variable
     # for git to use the wrapper script
-    ssh_wrapper = None
-    if key_file or ssh_opts:
-        ssh_wrapper = write_ssh_wrapper()
-        set_git_ssh(ssh_wrapper, key_file, ssh_opts)
-        module.add_cleanup_file(path=ssh_wrapper)
+    ssh_wrapper = write_ssh_wrapper()
+    set_git_ssh(ssh_wrapper, key_file, ssh_opts)
+    module.add_cleanup_file(path=ssh_wrapper)
 
-    # add the git repo's hostkey
-    if module.params['ssh_opts'] is not None:
-        if "-o StrictHostKeyChecking=no" not in module.params['ssh_opts']:
-            add_git_host_key(module, repo, accept_hostkey=module.params['accept_hostkey'])
-    else:
-        add_git_host_key(module, repo, accept_hostkey=module.params['accept_hostkey'])
     git_version_used = git_version(git_path, module)
 
     if depth is not None and git_version_used < LooseVersion('1.9.1'):
@@ -977,6 +1065,7 @@ def main():
     track_submodules = module.params['track_submodules']
 
     result.update(before=None)
+
     local_mods = False
     need_fetch = True
     if (dest and not os.path.exists(gitconfig)) or (not dest and not allow_clone):
@@ -1066,6 +1155,15 @@ def main():
             if diff:
                 result['diff'] = diff
 
+    if archive:
+        # Git archive is not supported by all git servers, so
+        # we will first clone and perform git archive from local directory
+        if module.check_mode:
+            result.update(changed=True)
+            module.exit_json(**result)
+
+        create_archive(git_path, module, dest, archive, version, repo, result)
+
     # cleanup the wrapper script
     if ssh_wrapper:
         try:
