@@ -1,27 +1,14 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 
-"""
-Ansible module to add authorized_keys for ssh logins.
-(c) 2012, Brad Olson <brado@movedbylight.com>
+# (c) 2012, Brad Olson <brado@movedbylight.com>
+# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
-This file is part of Ansible
+from __future__ import absolute_import, division, print_function
+__metaclass__ = type
 
-Ansible is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
 
-Ansible is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
-"""
-
-ANSIBLE_METADATA = {'metadata_version': '1.0',
+ANSIBLE_METADATA = {'metadata_version': '1.1',
                     'status': ['preview'],
                     'supported_by': 'core'}
 
@@ -92,6 +79,14 @@ options:
     default: "yes"
     choices: ["yes", "no"]
     version_added: "2.1"
+  comment:
+    description:
+      - Change the comment on the public key. Rewriting the comment is useful in
+        cases such as fetching it from GitHub or GitLab.
+      - If no comment is specified, the existing comment will be kept.
+    required: false
+    default: None
+    version_added: "2.4"
 author: "Ansible Core Team"
 '''
 
@@ -170,42 +165,42 @@ key_option:
   description: Key options related to the key.
   returned: success
   type: string
-  sameple: null
+  sample: null
 keyfile:
-  description: Path for authorzied key file.
+  description: Path for authorized key file.
   returned: success
   type: string
-  sameple: /home/user/.ssh/authorized_keys
+  sample: /home/user/.ssh/authorized_keys
 manage_dir:
   description: Whether this module managed the directory of the authorized key file.
   returned: success
   type: boolean
-  sameple: True
+  sample: True
 path:
   description: Alternate path to the authorized_keys file
   returned: success
   type: string
-  sameple: null
+  sample: null
 state:
   description: Whether the given key (with the given key_options) should or should not be in the file
   returned: success
   type: string
-  sameple: present
+  sample: present
 unique:
   description: Whether the key is unique
   returned: success
   type: boolean
-  sameple: false
+  sample: false
 user:
   description: The username on the remote host whose authorized_keys file will be modified
   returned: success
   type: string
-  sameple: user
+  sample: user
 validate_certs:
   description: This only applies if using a https url as the source of the keys. If set to C(no), the SSL certificates will not be validated.
   returned: success
   type: boolean
-  sameple: true
+  sample: true
 '''
 
 # Makes sure the public key line is present or absent in the user's .ssh/authorized_keys.
@@ -233,6 +228,7 @@ from ansible.module_utils.basic import AnsibleModule
 from ansible.module_utils.pycompat24 import get_exception
 from ansible.module_utils.urls import fetch_url
 
+
 class keydict(dict):
 
     """ a dictionary that maintains the order of keys as they are added
@@ -260,8 +256,8 @@ class keydict(dict):
     # http://stackoverflow.com/questions/2328235/pythonextend-the-dict-class
 
     def __init__(self, *args, **kw):
-        super(keydict,self).__init__(*args, **kw)
-        self.itemlist = list(super(keydict,self).keys())
+        super(keydict, self).__init__(*args, **kw)
+        self.itemlist = list(super(keydict, self).keys())
 
     def __setitem__(self, key, value):
         self.itemlist.append(key)
@@ -322,12 +318,12 @@ def keyfile(module, user, write=False, path=None, manage_dir=True):
             module.fail_json(msg="Either user must exist or you must provide full path to key file in check mode")
         module.fail_json(msg="Failed to lookup user %s: %s" % (user, str(e)))
     if path is None:
-        homedir    = user_entry.pw_dir
-        sshdir     = os.path.join(homedir, ".ssh")
-        keysfile   = os.path.join(sshdir, "authorized_keys")
+        homedir = user_entry.pw_dir
+        sshdir = os.path.join(homedir, ".ssh")
+        keysfile = os.path.join(sshdir, "authorized_keys")
     else:
-        sshdir     = os.path.dirname(path)
-        keysfile   = path
+        sshdir = os.path.dirname(path)
+        keysfile = path
 
     if not write:
         return keysfile
@@ -348,7 +344,7 @@ def keyfile(module, user, write=False, path=None, manage_dir=True):
         if not os.path.exists(basedir):
             os.makedirs(basedir)
         try:
-            f = open(keysfile, "w") #touches file so we can set ownership and perms
+            f = open(keysfile, "w")  # touches file so we can set ownership and perms
         finally:
             f.close()
         if module.selinux_enabled():
@@ -362,12 +358,13 @@ def keyfile(module, user, write=False, path=None, manage_dir=True):
 
     return keysfile
 
+
 def parseoptions(module, options):
     '''
     reads a string containing ssh-key options
     and returns a dictionary of those options
     '''
-    options_dict = keydict() #ordered dict
+    options_dict = keydict()  # ordered dict
     if options:
         # the following regex will split on commas while
         # ignoring those commas that fall within quotes
@@ -382,6 +379,7 @@ def parseoptions(module, options):
 
     return options_dict
 
+
 def parsekey(module, raw_key, rank=None):
     '''
     parses a key, which may or may not contain a list
@@ -400,9 +398,9 @@ def parsekey(module, raw_key, rank=None):
         'ssh-rsa',
     ]
 
-    options    = None   # connection options
-    key        = None   # encrypted key string
-    key_type   = None   # type of ssh key
+    options = None   # connection options
+    key = None   # encrypted key string
+    key_type = None   # type of ssh key
     type_index = None   # index of keytype in key string|list
 
     # remove comment yaml escapes
@@ -411,7 +409,7 @@ def parsekey(module, raw_key, rank=None):
     # split key safely
     lex = shlex.shlex(raw_key)
     lex.quotes = []
-    lex.commenters = '' #keep comment hashes
+    lex.commenters = ''  # keep comment hashes
     lex.whitespace_split = True
     key_parts = list(lex)
 
@@ -443,6 +441,7 @@ def parsekey(module, raw_key, rank=None):
 
     return (key, key_type, options, comment, rank)
 
+
 def readfile(filename):
 
     if not os.path.isfile(filename):
@@ -454,6 +453,7 @@ def readfile(filename):
     finally:
         f.close()
 
+
 def parsekeys(module, lines):
     keys = {}
     for rank_index, line in enumerate(lines.splitlines(True)):
@@ -467,10 +467,11 @@ def parsekeys(module, lines):
             keys[line] = (line, 'skipped', None, None, rank_index)
     return keys
 
+
 def writefile(module, filename, content):
 
     fd, tmp_path = tempfile.mkstemp('', 'tmp', os.path.dirname(filename))
-    f = open(tmp_path,"w")
+    f = open(tmp_path, "w")
 
     try:
         f.write(content)
@@ -480,6 +481,7 @@ def writefile(module, filename, content):
     f.close()
     module.atomic_move(tmp_path, filename)
 
+
 def serialize(keys):
     lines = []
     new_keys = keys.values()
@@ -509,24 +511,26 @@ def serialize(keys):
                 key_line = key[0]
             else:
                 key_line = "%s%s %s %s\n" % (option_str, key_type, keyhash, comment)
-        except:
+        except Exception:
             key_line = key
         lines.append(key_line)
     return ''.join(lines)
 
+
 def enforce_state(module, params):
     """
     Add or remove key.
     """
 
-    user        = params["user"]
-    key         = params["key"]
-    path        = params.get("path", None)
-    manage_dir  = params.get("manage_dir", True)
-    state       = params.get("state", "present")
+    user = params["user"]
+    key = params["key"]
+    path = params.get("path", None)
+    manage_dir = params.get("manage_dir", True)
+    state = params.get("state", "present")
     key_options = params.get("key_options", None)
-    exclusive   = params.get("exclusive", False)
-    error_msg   = "Error getting key from: %s"
+    exclusive = params.get("exclusive", False)
+    comment = params.get("comment", None)
+    error_msg = "Error getting key from: %s"
 
     # if the key is a url, request it and use it as key source
     if key.startswith("http"):
@@ -572,6 +576,9 @@ def enforce_state(module, params):
             # rank here is the rank in the provided new keys, which may be unrelated to rank in existing_keys
             parsed_new_key = (parsed_new_key[0], parsed_new_key[1], parsed_options, parsed_new_key[3], parsed_new_key[4])
 
+        if comment is not None:
+            parsed_new_key = (parsed_new_key[0], parsed_new_key[1], parsed_new_key[2], comment, parsed_new_key[4])
+
         matched = False
         non_matching_keys = []
 
@@ -587,7 +594,7 @@ def enforce_state(module, params):
                 matched = True
 
         # handle idempotent state=present
-        if state=="present":
+        if state == "present":
             keys_to_exist.append(parsed_new_key[0])
             if len(non_matching_keys) > 0:
                 for non_matching_key in non_matching_keys:
@@ -603,7 +610,7 @@ def enforce_state(module, params):
                 existing_keys[parsed_new_key[0]] = (parsed_new_key[0], parsed_new_key[1], parsed_new_key[2], parsed_new_key[3], total_rank)
                 do_write = True
 
-        elif state=="absent":
+        elif state == "absent":
             if not matched:
                 continue
             del existing_keys[parsed_new_key[0]]
@@ -620,35 +627,41 @@ def enforce_state(module, params):
     if do_write:
         filename = keyfile(module, user, do_write, path, manage_dir)
         new_content = serialize(existing_keys)
-        diff = {
-            'before_header': params['keyfile'],
-            'after_header': filename,
-            'before': existing_content,
-            'after': new_content,
-        }
+
+        diff = None
+        if module._diff:
+            diff = {
+                'before_header': params['keyfile'],
+                'after_header': filename,
+                'before': existing_content,
+                'after': new_content,
+            }
+            params['diff'] = diff
+
         if module.check_mode:
             module.exit_json(changed=True, diff=diff)
         writefile(module, filename, new_content)
         params['changed'] = True
-        params['diff'] = diff
     else:
         if module.check_mode:
             module.exit_json(changed=False)
 
     return params
 
+
 def main():
     module = AnsibleModule(
-        argument_spec = dict(
-            user        = dict(required=True, type='str'),
-            key         = dict(required=True, type='str'),
-            path        = dict(required=False, type='str'),
-            manage_dir  = dict(required=False, type='bool', default=True),
-            state       = dict(default='present', choices=['absent','present']),
-            key_options = dict(required=False, type='str'),
-            unique      = dict(default=False, type='bool'),
-            exclusive   = dict(default=False, type='bool'),
-            validate_certs = dict(default=True, type='bool'),
+        argument_spec=dict(
+            user=dict(required=True, type='str'),
+            key=dict(required=True, type='str'),
+            path=dict(required=False, type='str'),
+            manage_dir=dict(required=False, type='bool', default=True),
+            state=dict(default='present', choices=['absent', 'present']),
+            key_options=dict(required=False, type='str'),
+            unique=dict(default=False, type='bool'),
+            exclusive=dict(default=False, type='bool'),
+            comment=dict(required=False, default=None, type='str'),
+            validate_certs=dict(default=True, type='bool'),
         ),
         supports_check_mode=True
     )
@@ -656,5 +669,6 @@ def main():
     results = enforce_state(module, module.params)
     module.exit_json(**results)
 
+
 if __name__ == '__main__':
     main()
