@@ -28,18 +28,17 @@ import re
 import string
 import sys
 
-from ansible.compat.six import iteritems, string_types
-from ansible.compat.six.moves import shlex_quote
 from ansible import constants as C
 from ansible.errors import AnsibleError
+from ansible.module_utils.six import iteritems
+from ansible.module_utils.six.moves import shlex_quote
 from ansible.module_utils._text import to_bytes
+from ansible.module_utils.parsing.convert_bool import boolean
 from ansible.playbook.attribute import FieldAttribute
 from ansible.playbook.base import Base
+from ansible.plugins import get_plugin_class
 from ansible.utils.ssh_functions import check_for_controlpersist
 
-boolean = C.mk_boolean
-
-__all__ = ['PlayContext']
 
 try:
     from __main__ import display
@@ -47,49 +46,70 @@ except ImportError:
     from ansible.utils.display import Display
     display = Display()
 
+
+__all__ = ['PlayContext']
+
 # the magic variable mapping dictionary below is used to translate
 # host/inventory variables to fields in the PlayContext
 # object. The dictionary values are tuples, to account for aliases
 # in variable names.
 
 MAGIC_VARIABLE_MAPPING = dict(
-    connection       = ('ansible_connection',),
-    remote_addr      = ('ansible_ssh_host', 'ansible_host'),
-    remote_user      = ('ansible_ssh_user', 'ansible_user'),
-    port             = ('ansible_ssh_port', 'ansible_port'),
-    ssh_executable   = ('ansible_ssh_executable',),
-    accelerate_port  = ('ansible_accelerate_port',),
-    password         = ('ansible_ssh_pass', 'ansible_password'),
-    private_key_file = ('ansible_ssh_private_key_file', 'ansible_private_key_file'),
-    pipelining       = ('ansible_ssh_pipelining', 'ansible_pipelining'),
-    shell            = ('ansible_shell_type',),
-    network_os       = ('ansible_network_os',),
-    become           = ('ansible_become',),
-    become_method    = ('ansible_become_method',),
-    become_user      = ('ansible_become_user',),
-    become_pass      = ('ansible_become_password','ansible_become_pass'),
-    become_exe       = ('ansible_become_exe',),
-    become_flags     = ('ansible_become_flags',),
-    ssh_common_args  = ('ansible_ssh_common_args',),
-    docker_extra_args= ('ansible_docker_extra_args',),
-    sftp_extra_args  = ('ansible_sftp_extra_args',),
-    scp_extra_args   = ('ansible_scp_extra_args',),
-    ssh_extra_args   = ('ansible_ssh_extra_args',),
-    ssh_transfer_method = ('ansible_ssh_transfer_method',),
-    sudo             = ('ansible_sudo',),
-    sudo_user        = ('ansible_sudo_user',),
-    sudo_pass        = ('ansible_sudo_password', 'ansible_sudo_pass'),
-    sudo_exe         = ('ansible_sudo_exe',),
-    sudo_flags       = ('ansible_sudo_flags',),
-    su               = ('ansible_su',),
-    su_user          = ('ansible_su_user',),
-    su_pass          = ('ansible_su_password', 'ansible_su_pass'),
-    su_exe           = ('ansible_su_exe',),
-    su_flags         = ('ansible_su_flags',),
-    executable       = ('ansible_shell_executable',),
-    module_compression = ('ansible_module_compression',),
+    accelerate_port=('ansible_accelerate_port', ),
+
+    # base
+    connection=('ansible_connection', ),
+    module_compression=('ansible_module_compression', ),
+    shell=('ansible_shell_type', ),
+    executable=('ansible_shell_executable', ),
+    remote_tmp_dir=('ansible_remote_tmp', ),
+
+    # connection common
+    remote_addr=('ansible_ssh_host', 'ansible_host'),
+    remote_user=('ansible_ssh_user', 'ansible_user'),
+    password=('ansible_ssh_pass', 'ansible_password'),
+    port=('ansible_ssh_port', 'ansible_port'),
+    pipelining=('ansible_ssh_pipelining', 'ansible_pipelining'),
+    timeout=('ansible_ssh_timeout', 'ansible_timeout'),
+    private_key_file=('ansible_ssh_private_key_file', 'ansible_private_key_file'),
+
+    # networking modules
+    network_os=('ansible_network_os', ),
+    connection_user=('ansible_connection_user',),
+
+    # ssh TODO: remove
+    ssh_executable=('ansible_ssh_executable', ),
+    ssh_common_args=('ansible_ssh_common_args', ),
+    sftp_extra_args=('ansible_sftp_extra_args', ),
+    scp_extra_args=('ansible_scp_extra_args', ),
+    ssh_extra_args=('ansible_ssh_extra_args', ),
+    ssh_transfer_method=('ansible_ssh_transfer_method', ),
+
+    # docker TODO: remove
+    docker_extra_args=('ansible_docker_extra_args', ),
+
+    # become
+    become=('ansible_become', ),
+    become_method=('ansible_become_method', ),
+    become_user=('ansible_become_user', ),
+    become_pass=('ansible_become_password', 'ansible_become_pass'),
+    become_exe=('ansible_become_exe', ),
+    become_flags=('ansible_become_flags', ),
+
+    # deprecated
+    sudo=('ansible_sudo', ),
+    sudo_user=('ansible_sudo_user', ),
+    sudo_pass=('ansible_sudo_password', 'ansible_sudo_pass'),
+    sudo_exe=('ansible_sudo_exe', ),
+    sudo_flags=('ansible_sudo_flags', ),
+    su=('ansible_su', ),
+    su_user=('ansible_su_user', ),
+    su_pass=('ansible_su_password', 'ansible_su_pass'),
+    su_exe=('ansible_su_exe', ),
+    su_flags=('ansible_su_flags', ),
 )
 
+# TODO: needs to be configurable
 b_SU_PROMPT_LOCALIZATIONS = [
     to_bytes('Password'),
     to_bytes('암호'),
@@ -132,7 +152,7 @@ TASK_ATTRIBUTE_OVERRIDES = (
     'become_method',
     'become_flags',
     'connection',
-    'docker_extra_args',
+    'docker_extra_args',  # TODO: remove
     'delegate_to',
     'no_log',
     'remote_user',
@@ -140,6 +160,11 @@ TASK_ATTRIBUTE_OVERRIDES = (
 
 RESET_VARS = (
     'ansible_connection',
+    'ansible_user',
+    'ansible_host',
+    'ansible_port',
+
+    # TODO: ???
     'ansible_docker_extra_args',
     'ansible_ssh_host',
     'ansible_ssh_pass',
@@ -148,11 +173,9 @@ RESET_VARS = (
     'ansible_ssh_private_key_file',
     'ansible_ssh_pipelining',
     'ansible_ssh_executable',
-    'ansible_user',
-    'ansible_host',
-    'ansible_port',
 )
 
+
 class PlayContext(Base):
 
     '''
@@ -161,62 +184,73 @@ class PlayContext(Base):
     connection/authentication information.
     '''
 
+    # base
+    _module_compression = FieldAttribute(isa='string', default=C.DEFAULT_MODULE_COMPRESSION)
+    _shell = FieldAttribute(isa='string')
+    _executable = FieldAttribute(isa='string', default=C.DEFAULT_EXECUTABLE)
+
     # connection fields, some are inherited from Base:
     # (connection, port, remote_user, environment, no_log)
-    _docker_extra_args  = FieldAttribute(isa='string')
-    _remote_addr      = FieldAttribute(isa='string')
-    _password         = FieldAttribute(isa='string')
+    _remote_addr = FieldAttribute(isa='string')
+    _remote_tmp_dir = FieldAttribute(isa='string', default=C.DEFAULT_REMOTE_TMP)
+    _password = FieldAttribute(isa='string')
+    _timeout = FieldAttribute(isa='int', default=C.DEFAULT_TIMEOUT)
+    _connection_user = FieldAttribute(isa='string')
     _private_key_file = FieldAttribute(isa='string', default=C.DEFAULT_PRIVATE_KEY_FILE)
-    _timeout          = FieldAttribute(isa='int', default=C.DEFAULT_TIMEOUT)
-    _shell            = FieldAttribute(isa='string')
-    _network_os       = FieldAttribute(isa='string')
-    _connection_user  = FieldAttribute(isa='string')
-    _ssh_args         = FieldAttribute(isa='string', default=C.ANSIBLE_SSH_ARGS)
-    _ssh_common_args  = FieldAttribute(isa='string')
-    _sftp_extra_args  = FieldAttribute(isa='string')
-    _scp_extra_args   = FieldAttribute(isa='string')
-    _ssh_extra_args   = FieldAttribute(isa='string')
-    _ssh_executable   = FieldAttribute(isa='string', default=C.ANSIBLE_SSH_EXECUTABLE)
+    _pipelining = FieldAttribute(isa='bool', default=C.ANSIBLE_PIPELINING)
+
+    # networking modules
+    _network_os = FieldAttribute(isa='string')
+
+    # docker FIXME: remove these
+    _docker_extra_args = FieldAttribute(isa='string')
+
+    # ssh # FIXME: remove these
+    _ssh_executable = FieldAttribute(isa='string', default=C.ANSIBLE_SSH_EXECUTABLE)
+    _ssh_args = FieldAttribute(isa='string', default=C.ANSIBLE_SSH_ARGS)
+    _ssh_common_args = FieldAttribute(isa='string')
+    _sftp_extra_args = FieldAttribute(isa='string')
+    _scp_extra_args = FieldAttribute(isa='string')
+    _ssh_extra_args = FieldAttribute(isa='string')
     _ssh_transfer_method = FieldAttribute(isa='string', default=C.DEFAULT_SSH_TRANSFER_METHOD)
-    _connection_lockfd= FieldAttribute(isa='int')
-    _pipelining       = FieldAttribute(isa='bool', default=C.ANSIBLE_SSH_PIPELINING)
-    _accelerate       = FieldAttribute(isa='bool', default=False)
-    _accelerate_ipv6  = FieldAttribute(isa='bool', default=False, always_post_validate=True)
-    _accelerate_port  = FieldAttribute(isa='int', default=C.ACCELERATE_PORT, always_post_validate=True)
-    _executable       = FieldAttribute(isa='string', default=C.DEFAULT_EXECUTABLE)
-    _module_compression = FieldAttribute(isa='string', default=C.DEFAULT_MODULE_COMPRESSION)
+
+    # ???
+    _connection_lockfd = FieldAttribute(isa='int')
+
+    # accelerate FIXME: remove as soon as deprecation period expires
+    _accelerate = FieldAttribute(isa='bool', default=False)
+    _accelerate_ipv6 = FieldAttribute(isa='bool', default=False, always_post_validate=True)
+    _accelerate_port = FieldAttribute(isa='int', default=C.ACCELERATE_PORT, always_post_validate=True)
 
     # privilege escalation fields
-    _become           = FieldAttribute(isa='bool')
-    _become_method    = FieldAttribute(isa='string')
-    _become_user      = FieldAttribute(isa='string')
-    _become_pass      = FieldAttribute(isa='string')
-    _become_exe       = FieldAttribute(isa='string')
-    _become_flags     = FieldAttribute(isa='string')
-    _prompt           = FieldAttribute(isa='string')
+    _become = FieldAttribute(isa='bool')
+    _become_method = FieldAttribute(isa='string')
+    _become_user = FieldAttribute(isa='string')
+    _become_pass = FieldAttribute(isa='string')
+    _become_exe = FieldAttribute(isa='string', default=C.DEFAULT_BECOME_EXE)
+    _become_flags = FieldAttribute(isa='string', default=C.DEFAULT_BECOME_FLAGS)
+    _prompt = FieldAttribute(isa='string')
 
-    # backwards compatibility fields for sudo/su
-    _sudo_exe         = FieldAttribute(isa='string')
-    _sudo_flags       = FieldAttribute(isa='string')
-    _sudo_pass        = FieldAttribute(isa='string')
-    _su_exe           = FieldAttribute(isa='string')
-    _su_flags         = FieldAttribute(isa='string')
-    _su_pass          = FieldAttribute(isa='string')
+    # DEPRECATED: backwards compatibility fields for sudo/su
+    _sudo_exe = FieldAttribute(isa='string', default=C.DEFAULT_SUDO_EXE)
+    _sudo_flags = FieldAttribute(isa='string', default=C.DEFAULT_SUDO_FLAGS)
+    _sudo_pass = FieldAttribute(isa='string')
+    _su_exe = FieldAttribute(isa='string', default=C.DEFAULT_SU_EXE)
+    _su_flags = FieldAttribute(isa='string', default=C.DEFAULT_SU_FLAGS)
+    _su_pass = FieldAttribute(isa='string')
 
     # general flags
-    _verbosity        = FieldAttribute(isa='int', default=0)
-    _only_tags        = FieldAttribute(isa='set', default=set())
-    _skip_tags        = FieldAttribute(isa='set', default=set())
-    _check_mode       = FieldAttribute(isa='bool', default=False)
-    _force_handlers   = FieldAttribute(isa='bool', default=False)
-    _start_at_task    = FieldAttribute(isa='string')
-    _step             = FieldAttribute(isa='bool', default=False)
-    _diff             = FieldAttribute(isa='bool', default=C.DIFF_ALWAYS)
+    _verbosity = FieldAttribute(isa='int', default=0)
+    _only_tags = FieldAttribute(isa='set', default=set())
+    _skip_tags = FieldAttribute(isa='set', default=set())
+    _force_handlers = FieldAttribute(isa='bool', default=False)
+    _start_at_task = FieldAttribute(isa='string')
+    _step = FieldAttribute(isa='bool', default=False)
 
     # Fact gathering settings
-    _gather_subset    = FieldAttribute(isa='string', default=C.DEFAULT_GATHER_SUBSET)
-    _gather_timeout   = FieldAttribute(isa='string', default=C.DEFAULT_GATHER_TIMEOUT)
-    _fact_path        = FieldAttribute(isa='string', default=C.DEFAULT_FACT_PATH)
+    _gather_subset = FieldAttribute(isa='string', default=C.DEFAULT_GATHER_SUBSET)
+    _gather_timeout = FieldAttribute(isa='string', default=C.DEFAULT_GATHER_TIMEOUT)
+    _fact_path = FieldAttribute(isa='string', default=C.DEFAULT_FACT_PATH)
 
     def __init__(self, play=None, options=None, passwords=None, connection_lockfd=None):
 
@@ -225,10 +259,10 @@ class PlayContext(Base):
         if passwords is None:
             passwords = {}
 
-        self.password    = passwords.get('conn_pass','')
-        self.become_pass = passwords.get('become_pass','')
+        self.password = passwords.get('conn_pass', '')
+        self.become_pass = passwords.get('become_pass', '')
 
-        self.prompt      = ''
+        self.prompt = ''
         self.success_key = ''
 
         # a file descriptor to be used during locking operations
@@ -241,7 +275,6 @@ class PlayContext(Base):
         if play:
             self.set_play(play)
 
-
     def set_play(self, play):
         '''
         Configures this connection information instance with data from
@@ -273,6 +306,22 @@ class PlayContext(Base):
         if play.force_handlers is not None:
             self.force_handlers = play.force_handlers
 
+    def set_options_from_plugin(self, plugin):
+        # generic derived from connection plugin, temporary for backwards compat, in the end we should not set play_context properties
+
+        # get options for plugins
+        options = C.config.get_configuration_definitions(get_plugin_class(plugin), plugin._load_name)
+        for option in options:
+            if option:
+                flag = options[option].get('name')
+                if flag:
+                    setattr(self, flag, self.connection.get_option(flag))
+
+        # TODO: made irrelavent by above
+        # get ssh options
+        # for flag in ('ssh_common_args', 'docker_extra_args', 'sftp_extra_args', 'scp_extra_args', 'ssh_extra_args'):
+        #     setattr(self, flag, getattr(options, flag, ''))
+
     def set_options(self, options):
         '''
         Configures this connection information instance with data from
@@ -281,18 +330,17 @@ class PlayContext(Base):
         '''
 
         # privilege escalation
-        self.become        = options.become
+        self.become = options.become
         self.become_method = options.become_method
-        self.become_user   = options.become_user
+        self.become_user = options.become_user
 
-        self.check_mode = boolean(options.check)
+        self.check_mode = boolean(options.check, strict=False)
+        self.diff = boolean(options.diff, strict=False)
 
-        # get ssh options FIXME: make these common to all connections
-        for flag in ['ssh_common_args', 'docker_extra_args', 'sftp_extra_args', 'scp_extra_args', 'ssh_extra_args']:
-            setattr(self, flag, getattr(options,flag, ''))
-
-        # general flags (should we move out?)
-        for flag in ['connection','remote_user', 'private_key_file', 'verbosity', 'force_handlers', 'step', 'start_at_task', 'diff']:
+        #  general flags (should we move out?)
+        # for flag in ('connection', 'remote_user', 'private_key_file', 'verbosity', 'force_handlers', 'step', 'start_at_task', 'diff'):
+        #  should only be 'non plugin' flags
+        for flag in ('connection', 'remote_user', 'private_key_file', 'verbosity', 'force_handlers', 'step', 'start_at_task', 'diff'):
             attribute = getattr(options, flag, False)
             if attribute:
                 setattr(self, flag, attribute)
@@ -315,6 +363,10 @@ class PlayContext(Base):
         '''
         Sets attributes from the task if they are set, which will override
         those from the play.
+
+        :arg task: the task object with the parameters that were set on it
+        :arg variables: variables from inventory
+        :arg templar: templar instance if templating variables is needed
         '''
 
         new_info = self.copy()
@@ -349,7 +401,7 @@ class PlayContext(Base):
             # address, otherwise we default to connecting to it by name. This
             # may happen when users put an IP entry into their inventory, or if
             # they rely on DNS for a non-inventory hostname
-            for address_var in MAGIC_VARIABLE_MAPPING.get('remote_addr'):
+            for address_var in ('ansible_%s_host' % transport_var,) + MAGIC_VARIABLE_MAPPING.get('remote_addr'):
                 if address_var in delegated_vars:
                     break
             else:
@@ -358,7 +410,7 @@ class PlayContext(Base):
 
             # reset the port back to the default if none was specified, to prevent
             # the delegated host from inheriting the original host's setting
-            for port_var in MAGIC_VARIABLE_MAPPING.get('port'):
+            for port_var in ('ansible_%s_port' % transport_var,) + MAGIC_VARIABLE_MAPPING.get('port'):
                 if port_var in delegated_vars:
                     break
             else:
@@ -368,7 +420,7 @@ class PlayContext(Base):
                     delegated_vars['ansible_port'] = C.DEFAULT_REMOTE_PORT
 
             # and likewise for the remote user
-            for user_var in MAGIC_VARIABLE_MAPPING.get('remote_user'):
+            for user_var in ('ansible_%s_user' % transport_var,) + MAGIC_VARIABLE_MAPPING.get('remote_user'):
                 if user_var in delegated_vars and delegated_vars[user_var]:
                     break
             else:
@@ -399,9 +451,9 @@ class PlayContext(Base):
         # become legacy updates -- from commandline
         if not new_info.become_pass:
             if new_info.become_method == 'sudo' and new_info.sudo_pass:
-                setattr(new_info, 'become_pass', new_info.sudo_pass)
+                new_info.become_pass = new_info.sudo_pass
             elif new_info.become_method == 'su' and new_info.su_pass:
-                setattr(new_info, 'become_pass', new_info.su_pass)
+                new_info.become_pass = new_info.su_pass
 
         # become legacy updates -- from inventory file (inventory overrides
         # commandline)
@@ -433,7 +485,7 @@ class PlayContext(Base):
                 if connection_type in delegated_vars:
                     break
             else:
-                remote_addr_local  = new_info.remote_addr in C.LOCALHOST
+                remote_addr_local = new_info.remote_addr in C.LOCALHOST
                 inv_hostname_local = delegated_vars.get('inventory_hostname') in C.LOCALHOST
                 if remote_addr_local and inv_hostname_local:
                     setattr(new_info, 'connection', 'local')
@@ -444,16 +496,14 @@ class PlayContext(Base):
         # this ensures any become settings are obeyed correctly
         # we store original in 'connection_user' for use of network/other modules that fallback to it as login user
         if new_info.connection == 'local':
-            new_info.connection_user = new_info.remote_user
+            if not new_info.connection_user:
+                new_info.connection_user = new_info.remote_user
             new_info.remote_user = pwd.getpwuid(os.getuid()).pw_name
 
         # set no_log to default if it was not previouslly set
         if new_info.no_log is None:
             new_info.no_log = C.DEFAULT_NO_LOG
 
-        # set become defaults if not previouslly set
-        task.set_become_defaults(new_info.become, new_info.become_method, new_info.become_user)
-
         if task.always_run:
             display.deprecated("always_run is deprecated. Use check_mode = no instead.", version="2.4", removed=False)
             new_info.check_mode = False
@@ -462,13 +512,15 @@ class PlayContext(Base):
         if task.check_mode is not None:
             new_info.check_mode = task.check_mode
 
+        if task.diff is not None:
+            new_info.diff = task.diff
 
         return new_info
 
     def make_become_cmd(self, cmd, executable=None):
         """ helper function to create privilege escalation commands """
 
-        prompt      = None
+        prompt = None
         success_key = None
         self.prompt = None
 
@@ -477,8 +529,8 @@ class PlayContext(Base):
             if not executable:
                 executable = self.executable
 
-            becomecmd   = None
-            randbits    = ''.join(random.choice(string.ascii_lowercase) for x in range(32))
+            becomecmd = None
+            randbits = ''.join(random.choice(string.ascii_lowercase) for x in range(32))
             success_key = 'BECOME-SUCCESS-%s' % randbits
             success_cmd = shlex_quote('echo %s; %s' % (success_key, cmd))
 
@@ -488,18 +540,18 @@ class PlayContext(Base):
                 command = success_cmd
 
             # set executable to use for the privilege escalation method, with various overrides
-            exe = self.become_exe or \
-                  getattr(self, '%s_exe' % self.become_method, None) or \
-                  C.DEFAULT_BECOME_EXE or \
-                  getattr(C, 'DEFAULT_%s_EXE' % self.become_method.upper(), None) or \
-                  self.become_method
+            exe = self.become_method
+            for myexe in (getattr(self, '%s_exe' % self.become_method, None), self.become_exe):
+                if myexe:
+                    exe = myexe
+                    break
 
             # set flags to use for the privilege escalation method, with various overrides
-            flags = self.become_flags or \
-                    getattr(self, '%s_flags' % self.become_method, None) or \
-                    C.DEFAULT_BECOME_FLAGS or \
-                    getattr(C, 'DEFAULT_%s_FLAGS' % self.become_method.upper(), None) or \
-                    ''
+            flags = ''
+            for myflag in (getattr(self, '%s_flags' % self.become_method, None), self.become_flags):
+                if myflag is not None:
+                    flags = myflag
+                    break
 
             if self.become_method == 'sudo':
                 # If we have a password, we run sudo with a randomly-generated
@@ -515,11 +567,10 @@ class PlayContext(Base):
                 # force quick error if password is required but not supplied, should prevent sudo hangs.
                 if self.become_pass:
                     prompt = '[sudo via ansible, key=%s] password: ' % randbits
-                    becomecmd = '%s %s -p "%s" -u %s %s' % (exe,  flags.replace('-n',''), prompt, self.become_user, command)
+                    becomecmd = '%s %s -p "%s" -u %s %s' % (exe, flags.replace('-n', ''), prompt, self.become_user, command)
                 else:
                     becomecmd = '%s %s -u %s %s' % (exe, flags, self.become_user, command)
 
-
             elif self.become_method == 'su':
 
                 # passing code ref to examine prompt as simple string comparisson isn't good enough with su
@@ -535,7 +586,7 @@ class PlayContext(Base):
 
             elif self.become_method == 'pbrun':
 
-                prompt='Password:'
+                prompt = 'Password:'
                 becomecmd = '%s %s -u %s %s' % (exe, flags, self.become_user, success_cmd)
 
             elif self.become_method == 'ksu':
@@ -573,7 +624,7 @@ class PlayContext(Base):
                 if self.become_user:
                     flags += ' -u %s ' % self.become_user
 
-                #FIXME: make shell independent
+                # FIXME: make shell independent
                 becomecmd = '%s %s echo %s && %s %s env ANSIBLE=true %s' % (exe, flags, success_key, exe, flags, cmd)
 
             elif self.become_method == 'dzdo':
@@ -585,6 +636,13 @@ class PlayContext(Base):
                 else:
                     becomecmd = '%s -u %s %s' % (exe, self.become_user, command)
 
+            elif self.become_method == 'pmrun':
+
+                exe = self.become_exe or 'pmrun'
+
+                prompt = 'Enter UPM user password:'
+                becomecmd = '%s %s %s' % (exe, flags, shlex_quote(command))
+
             else:
                 raise AnsibleError("Privilege escalation method not found: %s" % self.become_method)
 
@@ -636,4 +694,3 @@ class PlayContext(Base):
             self.connection = conn_type
 
         return self._attributes['connection']
-
